/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.freemarkerdor;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.container.httpserver.HttpServerFactory;
import com.sun.jersey.api.core.PackagesResourceConfig;
import com.sun.jersey.api.core.ResourceConfig;
import com.sun.jersey.api.view.Viewable;
import com.sun.net.httpserver.HttpServer;
import cz.xelfi.quoridor.webidor.resources.Quoridor;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.NewCookie;
import javax.ws.rs.core.Response;
import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;
import org.w3c.dom.Document;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Path("/")
public final class UI {
    private static WebResource base;

    @Context
    private HttpHeaders headers;
    private String user;

    public UI() {
    }

    private Viewable checkLogin() {
        if (headers.getCookies().containsKey("login")) {
            user = headers.getCookies().get("login").getValue();
            return null;
        }
        return new Viewable("login.fmt", null);
    }

    @POST
    @Path("login")
    @Produces(MediaType.TEXT_HTML)
    public Response login(
        @FormParam("name") String name, @FormParam("password") String password
    ) throws Exception {
        File f = new File(new File(new File(System.getProperty("user.home")), ".quoridor"), "passwd");
        Properties p = new Properties();
        p.load(new FileInputStream(f));
        if (name != null && password.equals(p.getProperty(name))) {
            return Response.seeOther(new URI("/")).cookie(new NewCookie("login", name)).entity(welcomeImpl()).build();
        } else {
            Viewable v = new Viewable("login.fmt", "Invalid name or password: " + name);
            return Response.status(1).entity(v).build();
        }
    }

    @GET
    @Produces(MediaType.TEXT_HTML)
    public Viewable welcome() throws JSONException {
        Viewable v = checkLogin();
        if (v != null) {
            return v;
        }
        return welcomeImpl();
    }

    @GET
    @Path("games/{id}/")
    @Produces(MediaType.TEXT_HTML)
    public Viewable board(@PathParam("id") String id) throws JSONException {
        Viewable v = checkLogin();
        if (v != null) {
            return v;
        }
        Map<?,?> obj = (Map<?,?>)convert(base.path("games").path(id).accept(MediaType.APPLICATION_JSON_TYPE).get(JSONObject.class));


        return new Viewable("game.fmt", obj);
    }

    @GET
    @Path("games/{id}/move")
    @Produces(MediaType.TEXT_HTML)
    public Viewable move(
        @PathParam("id") String id,
        @QueryParam("type") String type,
        @QueryParam("direction") String direction,
        @QueryParam("direction-next") @DefaultValue("") String directionNext,
        @QueryParam("column") @DefaultValue("") String column,
        @QueryParam("row") @DefaultValue("") String row
    ) throws JSONException {
        Viewable v = checkLogin();
        if (v != null) {
            return v;
        }
        WebResource wr = base.path("games").path(id).queryParam("player", user);
        if (type.equals("fence")) {
            wr.queryParam("move", direction.charAt(0) + column + row).put();
            return board(id);
        }
        if (type.equals("move")) {
            wr.queryParam("move", direction + directionNext).put();
            return board(id);
        }
        return board(id);
    }

    @GET
    @Path("games/create")
    @Produces(MediaType.TEXT_HTML)
    public Viewable create(
        @QueryParam("white") String white,
        @QueryParam("black") String black
    ) throws JSONException {
        Viewable v = checkLogin();
        if (v != null) {
            return v;
        }
        Object obj = convert(
            base.path("games").queryParam("white", white).
            queryParam("black", black).post(JSONObject.class)
        );
        Map<?,?> map = (Map<?,?>)obj;
        String id = (String)map.get("id");
        return board(id);
    }


    private static Object convert(Object obj) throws JSONException {
        if (obj instanceof JSONArray) {
            JSONArray arr = (JSONArray)obj;
            final int length = arr.length();
            List<Object> res = new ArrayList<Object>(length);
            for (int i = 0; i < length; i++) {
                res.add(convert(arr.get(i)));
            }
            return res;
        } else if (obj instanceof JSONObject) {
            JSONObject json = (JSONObject)obj;
            Map<Object,Object> map = new HashMap<Object,Object>(json.length() * 2 / 3);
            for (Iterator it = json.keys(); it.hasNext();) {
                String key = (String)it.next();
                map.put(key, convert(json.get(key)));
            }
            return map;
        } else {
            return obj;
        }
    }

    //
    // start the server
    //

    public static void main(String[] args) throws Exception {
        HttpServer api = Quoridor.start(9998);
        Client client = new Client();
        base = client.resource(new URI("http://localhost:9998/api/"));

        HttpServer s = start(9997);
        System.out.println(
            "Quoridor started at port 9997\n" + "Hit enter to stop it..."
        );
        System.in.read();
        s.stop(0);
        System.exit(0);
    }

    static HttpServer start(int port) throws IOException {
        final String baseUri = "http://localhost:" + port + "/";
        ResourceConfig rc = new PackagesResourceConfig("cz.xelfi.quoridor.freemarkerdor");
        HttpServer server = HttpServerFactory.create(baseUri, rc);
        server.start();
        return server;
    }

    private Viewable welcomeImpl() throws JSONException {
        final Object got = base.path("games").accept(MediaType.TEXT_XML_TYPE).get(Document.class);
        Map<?,?> obj = (Map<?, ?>)convert(got);
        return new Viewable("index.fmt", obj);
    }

}
